package core.services;

import core.api.models.properties.PropertyInfo;
import core.entities.PropertiesEntity;
import core.repositories.PropertyRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.parsing.PropertyEntry;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
public class PropertyService {

    @Autowired
    private PropertyRepository propertyRepository;



    public void init() {
        PropertiesEntity registrationEntity = new PropertiesEntity(PropertyName.REGISTRATION_PROPERTY_NAME.name(), true);
        if(!propertyRepository.existsByName(registrationEntity.getName())) {
            propertyRepository.save(registrationEntity);
        }
    }

    public Optional<PropertyInfo> getProperty(PropertyName propertyName) {
        Optional<PropertiesEntity> property = propertyRepository.findByName(propertyName.name());
        return property.map(PropertyInfo::new);
    }

    public Optional<PropertiesEntity> getPropertyEntityByName(String name) {
        return propertyRepository.findByName(name);
    }

    public boolean isPropertyEnabled(PropertyName propertyName) {
        Optional<PropertiesEntity> property = propertyRepository.findByName(propertyName.name());
        return property.map(PropertiesEntity::isStatus).orElseGet(() -> getDefaultBehaivior(propertyName));
    }

    public boolean getDefaultBehaivior(PropertyName propertyName) {
        return switch (propertyName) {
            case REGISTRATION_PROPERTY_NAME -> true;
        };
    }

    public void savePropertyEntity(PropertiesEntity property) {
        propertyRepository.save(property);
    }

    public enum PropertyName {
        REGISTRATION_PROPERTY_NAME
    }
}
