package core.services;

import core.api.models.timemanager.taskgroup.TaskgroupEntityInfo;
import core.api.models.timemanager.taskgroup.TaskgroupFieldInfo;
import core.entities.User;
import core.entities.UserRole;
import core.entities.timemanager.Taskgroup;
import core.repositories.UserRepository;
import core.repositories.timemanager.TaskgroupRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.config.Task;
import org.springframework.security.core.parameters.P;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;

@Service
public class TaskgroupService {

    private final TaskgroupRepository taskgroupRepository;
    private final UserRepository userRepository;
    public TaskgroupService(@Autowired  TaskgroupRepository taskgroupRepository,
                            @Autowired UserRepository userRepository) {
        this.taskgroupRepository = taskgroupRepository;
        this.userRepository = userRepository;
    }

    public PermissionResult<Taskgroup> getTaskgroupByIDAndUsername(long taskgroupID, String username) {
        Optional<Taskgroup> taskgroup = taskgroupRepository.findById(taskgroupID);
        return taskgroup.map(value -> new PermissionResult<>(value, value.getUser().getUsername().equals(username))).orElseGet(() ->
                new PermissionResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public ServiceResult<Taskgroup> addTaskgroup(TaskgroupFieldInfo taskData, String username) {
        Optional<User> user = userRepository.findByUsername(username);
        if(user.isEmpty()) {
            throw new NoSuchElementException();
        }

        if(!taskgroupRepository.existsByTaskgroupNameAndUser(taskData.getName(), user.get())) {

            Taskgroup taskgroup = new Taskgroup(taskData.getName(), user.get());
            taskgroupRepository.save(taskgroup);
            return new ServiceResult<>(taskgroup);
        } else {
            return new ServiceResult<>(ServiceExitCode.ENTITY_ALREADY_EXIST);
        }
    }

    public ServiceExitCode editTaskgroup(Taskgroup taskgroup, TaskgroupFieldInfo taskgroupFieldInfo) {
        if(!taskgroup.getTaskgroupName().equals(taskgroupFieldInfo.getName())) {
            //Check if another taskgroup with the new name is already existent for the user of the taskgroup
            if(!taskgroupRepository.existsByTaskgroupNameAndUser(taskgroupFieldInfo.getName(), taskgroup.getUser())) {
                taskgroup.setTaskgroupName(taskgroupFieldInfo.getName());
                taskgroupRepository.save(taskgroup);
                return ServiceExitCode.OK;
            } else {
                return ServiceExitCode.ENTITY_ALREADY_EXIST;
            }
        } else {
            return ServiceExitCode.OK;
        }
    }

    public void deleteTaskgroup(Taskgroup taskgroup) {
        taskgroupRepository.delete(taskgroup);
    }

    public List<Taskgroup> getTaskgroupsByUser(String username) {
        return taskgroupRepository.findAllByUser(username);
    }

    public void deleteTaskgroupByUser(User user) {
        taskgroupRepository.deleteAllByUser(user);
    }
}
